﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class InputController : MonoBehaviour
{
    public delegate void DirectionHeldDelegate(Vector2 direction);
    public delegate void DirectionUpDelegate(Vector2 direction);
    public delegate void InteractDownDelegate();
    public delegate void InteractUpDelegate();

    [Tooltip("Whether or not we are currently listening for input")]
    [SerializeField]
    protected bool allowInput = true;

    protected DirectionHeldDelegate directionHeldDelegate;

    protected DirectionUpDelegate directionUpDelegate;

    protected InteractDownDelegate interactDownDelegate;

    protected InteractUpDelegate interactUpDelegate;

    public DirectionHeldDelegate OnDirectionHeld
    {
        set
        {
            directionHeldDelegate = value;
        }
    }

    public bool AllowInput
    {
        set
        {
            allowInput = value;
        }
    }

    public DirectionUpDelegate OnDirectionUp
    {
        set
        {
            directionUpDelegate = value;
        }
    }

    public InteractDownDelegate OnInteractDown
    {
        set
        {
            interactDownDelegate = value;
        }
    }

    public InteractUpDelegate OnInteractUp
    {
        set
        {
            interactUpDelegate = value;
        }
    }

    private void Update()
    {
        if (allowInput)
        {
            DetectDirectionsHeld();
            DetectDirectionsUp();
            DetectInteractDown();
            DetectInteractUp();
        }
    }

    protected void DetectDirectionsHeld()
    {
        if( Input.GetKey(KeyCode.A) || Input.GetKey(KeyCode.LeftArrow))
        {
            directionHeldDelegate?.Invoke( new Vector2(-1f,0f));
        }
        else if (Input.GetKey(KeyCode.D) || Input.GetKey(KeyCode.RightArrow))
        {
            directionHeldDelegate?.Invoke(new Vector2(1f, 0f));
        }

        if (Input.GetKey(KeyCode.W) || Input.GetKey(KeyCode.UpArrow))
        {
            directionHeldDelegate?.Invoke(new Vector2(0f, 1f));
        }
        else if (Input.GetKey(KeyCode.S) || Input.GetKey(KeyCode.DownArrow))
        {
            directionHeldDelegate?.Invoke(new Vector2(0f, -1f));
        }
    }

    protected void DetectDirectionsUp()
    {
        if (Input.GetKeyUp(KeyCode.A) || Input.GetKeyUp(KeyCode.LeftArrow))
        {
            directionUpDelegate?.Invoke(new Vector2(-1f, 0f));
        }
        else if (Input.GetKeyUp(KeyCode.D) || Input.GetKeyUp(KeyCode.RightArrow))
        {
            directionUpDelegate?.Invoke(new Vector2(1f, 0f));
        }

        if (Input.GetKeyUp(KeyCode.W) || Input.GetKeyUp(KeyCode.UpArrow))
        {
            directionUpDelegate?.Invoke(new Vector2(0f, 1f));
        }
        else if (Input.GetKeyUp(KeyCode.S) || Input.GetKeyUp(KeyCode.DownArrow))
        {
            directionUpDelegate?.Invoke(new Vector2(0f, -1f));
        }
    }

    protected void DetectInteractDown()
    {
        if (Input.GetKeyDown(KeyCode.Space))
        {
            interactDownDelegate?.Invoke();
        }
    }

    protected void DetectInteractUp()
    {
        if (Input.GetKeyUp(KeyCode.Space))
        {
            interactUpDelegate?.Invoke();
        }
    }
}
