using System.Collections;
using System.Collections.Generic;
using UnityEngine;

[RequireComponent(typeof(Animator))]
public class DestructibleModifierAnimator : DestructibleModifier
{
    public bool setsHurtTrigger = true; //"hurt" trigger
    public bool setsHealedTrigger = true; //"healed" trigger
    public bool setsDeadBool = true; //"dead" bool

    protected bool playedDeathAnimation = false;
    protected bool shouldBeDestroyed = false;
    
    protected Animator animator;

    public override void Awake() 
    {
        base.Awake();
        animator = GetComponent<Animator>();
    }

    //the animator should call this when we're ready to remove this
    public void MarkReadyForDestroy()
    {
        shouldBeDestroyed = true;
    }
    
    public void Update()
    {
        //if the animator told us to be destroyed, have the destructible try death again (we won't block it)
        if ( shouldBeDestroyed )
        {
            destructible.Die();
        }
    }
    
    public override float ModifyHitPointsChange( float amount, Destructor source )
    {
        if ( amount < 0.0f && setsHurtTrigger )
        {
            animator.SetTrigger( "hurt" );
        }
        else if ( amount > 0.0f && setsHealedTrigger )
        {
            animator.SetTrigger( "healed" );
        }

        return base.ModifyHitPointsChange( amount, source );
    }

    //don't allow death normally, play the animation instead,
    //unless we've already played and completed the animation.
    public override bool ModifyShouldDie( bool shouldDie, Destructor source )
    {
        if ( shouldDie )
        {
            if ( !playedDeathAnimation && setsDeadBool )
            {
                animator.SetBool( "dead", true );
            }
        }

        return base.ModifyShouldDie( shouldDie, source ) && shouldBeDestroyed;
    }
}
