﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class AmbientAudioPlayer : MonoBehaviour
{
    [Tooltip("We'll play these sounds.")]
    [SerializeField]
    protected AudioClip[] sounds;

    [Tooltip("At minimum it will be this long between sounds.")]
    [SerializeField]
    protected float minimumTimeBetweenSounds = 1.0f;

    [Tooltip("At maximum it will be this long between sounds.")]
    [SerializeField]
    protected float maximumTimeBetweenSounds = 1.0f;

    [Tooltip("If true, we'll start playing sounds automatically on start.")]
    [SerializeField]
    protected bool beginOnStart = true;

    [Tooltip("If true, we'll set our Z to be 0 at awake.")]
    [SerializeField]
    protected bool forceZeroZ = true;

    protected AudioSource audioSource;

    public bool IsPlayingSounds
    {
        get;
        protected set;
    }

    public void Awake()
    {
        audioSource = GetComponent<AudioSource>();

        if ( forceZeroZ )
        {
            Vector3 pos = transform.position;
            pos.z = 0.0f;
            transform.position = pos;
        }
    }

    public void Start()
    {
        if ( beginOnStart )
        {
            BeginPlayingSounds();
        }
    }

    public void BeginPlayingSounds()
    {
        if ( IsPlayingSounds )
        {
            return;
        }
        IsPlayingSounds = true;
        ScheduleNextSound();
    }

    public void StopPlayingSounds()
    {
        IsPlayingSounds = false;
    }

    public void PlaySound()
    {
        if ( !IsPlayingSounds )
        {
            return;
        }

        Sounds.Play( sounds, audioSource );
        ScheduleNextSound();
    }

    protected void ScheduleNextSound()
    {
        Invoke( "PlaySound", Random.Range( minimumTimeBetweenSounds, maximumTimeBetweenSounds ) );
    }
}
