using System.Collections.Generic;
using UnityEngine;

public class Sounds : MonoBehaviour
{
    private static Dictionary<int,int> audioBuckets = new Dictionary<int,int>();
    private static GameObject oneTimeSoundPrefab;

    public static void Play( AudioClip[] sounds )
    {
        Play( sounds, null );
    }

    public static void Play( AudioClip sound )
    {
        Play( sound, null );
    }

    public static void Play( AudioClip[] sounds, AudioSource audioSource )
    {
        if ( sounds.Length <= 0 )
        {
            return;
        }
                
        if ( sounds.Length <= 1 )
        {
            Play( sounds[ 0 ] );
        }
        else
        {
            AudioClip leastCommonSound = sounds[ Random.Range( 0, sounds.Length ) ];
            int leastCommonCount = audioBuckets.ContainsKey( leastCommonSound.GetInstanceID() ) ? audioBuckets[ leastCommonSound.GetInstanceID() ] : 0;
            
            for ( int soundIndex = 0; soundIndex < sounds.Length; soundIndex++ )
            {
                int audioID = sounds[ soundIndex ].GetInstanceID();
                int count = 0;
                if ( audioBuckets.ContainsKey( audioID ) )
                {
                    count = audioBuckets[ audioID ];
                }
                
                if ( count < leastCommonCount )
                {
                    leastCommonSound = sounds[ soundIndex ];
                    leastCommonCount = count;
                }
            }
            
            audioBuckets[ leastCommonSound.GetInstanceID() ] = leastCommonCount + 1;
            Play( leastCommonSound, audioSource );
        }
    }
    
    public static void Play( AudioClip sound, AudioSource audioSource )
    {
        if ( audioSource == null )
        {
            if ( oneTimeSoundPrefab == null )
            {
                oneTimeSoundPrefab = Resources.Load<GameObject>( "OneTimeSound" );

                if ( oneTimeSoundPrefab == null )
                {
                    oneTimeSoundPrefab = new GameObject();
                    oneTimeSoundPrefab.AddComponent<AudioSource>();
                    oneTimeSoundPrefab.AddComponent<OneTimeSound>();
                }
            }
            
            GameObject obj = Object.Instantiate<GameObject>( oneTimeSoundPrefab );
            obj.GetComponent<OneTimeSound>().Play( sound );
        }
        else
        {
            audioSource.clip = sound;
            audioSource.Play();
        }
    }
    
    //warning - inefficient. also, only stops OneTimeSound's, won't stop anything played by a different AudioSource
    public static void StopAll()
    {
        OneTimeSound[] sounds = Object.FindObjectsOfType<OneTimeSound>();
        for ( int soundIndex = 0; soundIndex < sounds.Length; soundIndex++ )
        {
            Object.Destroy( sounds[ soundIndex ].gameObject );
        }
    }
}
