﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class Level : MonoBehaviour
{
    [Tooltip("Points to the player's start location, they will be spawned here. For convenience it's destroyed later.")]
    [SerializeField]
    protected Transform playerStartLocation;

    [Tooltip("Points to the knight's start location, they will be spawned here. For convenience it's destroyed later.")]
    [SerializeField]
    protected Transform knightStartLocation;

    [Tooltip("Z sorts all of its children by their Y position divided by this. If it's negative or 0, won't z sort.")]
    [SerializeField]
    protected float zSortDivisor = 10.0f;

    [Tooltip("We will Z sort every transform that you pass in here.")]
    [SerializeField]
    protected List<Transform> zSortedTransforms;

    protected Dictionary<Transform,float> initialZs;

    protected PlayerController player;
    protected KnightController knight;

    public bool WasWon
    {
        get;
        protected set;
    }

    public bool WasLost
    {
        get;
        protected set;
    }

    public void Awake()
    {
        initialZs = new Dictionary<Transform, float>();
    }

    public void Start()
    {
        if ( World.Instance == null || Game.Instance == null )
        {
            string levelName = SceneLoader.GetLoadedSceneName();
            try
            {
                int levelNumber = int.Parse( levelName.Substring( "Level".Length ) );
                World.LevelToLoad = levelNumber;
            }
            catch( System.Exception err )
            {
                Debug.LogWarning( "Name your scenes right yo. need to start with 'Level'.\n" + err );
            }
            
            SceneLoader.LoadScene( SceneNames.BOOTSTRAPPER_SCENE );
        }
    }

    public void Initialize( PlayerController newPlayer, KnightController newKnight )
    {
        player = newPlayer;
        knight = newKnight;

        Vector3 playerStartPos = playerStartLocation.transform.position;
        playerStartPos.z = player.transform.position.z;
        player.transform.position = playerStartPos;

        Vector3 knightStartPos = knightStartLocation.transform.position;
        knightStartPos.z = knight.transform.position.z;
        knight.transform.position = knightStartPos;

        zSortedTransforms.Add( player.transform );
        zSortedTransforms.Add( knight.transform );

        Destroy( playerStartLocation.gameObject );
        Destroy( knightStartLocation.gameObject );

        GameCamera cam = GameObject.FindObjectOfType<GameCamera>();
        cam.target = player.transform;
    }

    public void LateUpdate()
    {
        // if ( zSortDivisor > 0.0f )
        // {
        //     foreach( Transform t in zSortedTransforms )
        //     {
        //         if ( t != null )
        //         {
        //             if ( !initialZs.ContainsKey( t ) )
        //             {
        //                 initialZs[t] = t.position.z;
        //             }

        //             Vector3 tPos = t.position;
        //             tPos.z = initialZs[t] + tPos.y / zSortDivisor;
        //             t.position = tPos;
        //         }
        //     }
        // }
    }

    public void Win()
    {
        if ( WasWon || WasLost )
        {
            return;
        }

        WasWon = true;
        player.Win();
        knight.Win();
        Star.ScoreCurrentStars();

        //I put these clips in the world so I don't have to link the sounds in every level :'-(
        Sounds.Play( World.Instance.WinSounds );
    }

    public void Lose()
    {
        if ( WasWon || WasLost )
        {
            return;
        }

        if(player)
        {
            player.Lose();
        }

        WasLost = true;
        Sounds.Play( World.Instance.LoseSounds );
    }
}
