﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class Star : MonoBehaviour
{
    [Tooltip("Our animator for playing effects.")]
    [SerializeField]
    protected Animator animator;

    [Tooltip("We'll toggle this bool once picked up.")]
    [SerializeField]
    protected string animatorBoolPickedUp = "pickedUp";

    [Tooltip("We play one of these sound effects when a star is picked up.")]
    [SerializeField]
    protected AudioClip[] pickUpSounds;

    public const string STAR_COUNT_KEY_PREFIX = "starCount.";

    public bool WasPickedUp
    {
        get;
        protected set;
    }

    public static int GetMaximumStarCount( int level )
    {
        return PlayerPrefs.GetInt( STAR_COUNT_KEY_PREFIX + level, 0 );
    }

    public static void SetMaximumStarCount( int level, int val )
    {
        PlayerPrefs.SetInt( STAR_COUNT_KEY_PREFIX + level, val );
    }

    public static void ScoreCurrentStars()
    {
        int levelIndex = World.Instance.CurrentLevelIndex;
        SetMaximumStarCount( levelIndex, Mathf.Max( GetMaximumStarCount( levelIndex ), CurrentStarCount ) );
    }

    public static int CurrentStarCount
    {
        get;
        protected set;
    }

    public void Start()
    {  
        //this is some haaaaaaaaack hack
        CurrentStarCount = 0;
    }

    public void OnTriggerEnter2D( Collider2D coll )
    {
        if ( WasPickedUp )
        {
            return;
        }

        KnightController knight = coll.GetComponent<KnightController>();
        if ( knight != null )
        {
            WasPickedUp = true;

            animator.SetBool( animatorBoolPickedUp, true );
            CurrentStarCount++;

            Sounds.Play( pickUpSounds );
            knight.PickUpStar();
        }
    }
}
