﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using TMPro;

public class GameUI : MonoBehaviour
{
    [Tooltip("Text that shows the trial number at the start of the level.")]
    [SerializeField]
    protected TextMeshProUGUI levelIntroText;

    [Tooltip("We'll tell all of these stars to play as we animate them.")]
    [SerializeField]
    protected StarUI[] starUIs;

    [Tooltip("The animator that controls win/lose game overlays.")]
    [SerializeField]
    protected Animator animator;

    [Tooltip("Animator bool for when we've won.")]
    [SerializeField]
    protected string animatorBoolWon = "won";

    [Tooltip("Animator bool for when we've lost.")]
    [SerializeField]
    protected string animatorBoolLost = "lost";

    [Tooltip("Sounds played for clicking to go to the next level.")]
    [SerializeField]
    protected AudioClip[] nextLevelButtonSounds;

    [Tooltip("Sounds played for clicking to restart the level.")]
    [SerializeField]
    protected AudioClip[] restartLevelButtonSounds;

    [Tooltip("Sounds played for clicking to go to level select.")]
    [SerializeField]
    protected AudioClip[] levelSelectButtonSounds;

    public void Start()
    {
        if ( World.Instance == null )
        {
            return;
        }

        if (levelIntroText != null)
        {
            levelIntroText.text = "Trial " + (World.Instance.CurrentLevelIndex + 1) + " of " + World.Instance.LevelCount;
        }
    }

    public void Update()
    {
        if ( animator == null || Game.Instance == null )
        {
            return;
        }

        animator.SetBool( animatorBoolWon, Game.Instance.WonLevel );
        animator.SetBool( animatorBoolLost, Game.Instance.LostLevel );
    }

    public void GoToNextLevel()
    {
        Sounds.Play( nextLevelButtonSounds );
        Game.Instance.GoToNextLevel();
    }

    public void RestartLevel()
    {
        Sounds.Play( restartLevelButtonSounds );
        Game.Instance.RestartLevel();
    }

    public void GoToLevelSelect()
    {
        Sounds.Play( levelSelectButtonSounds );
        MusicController.Instance.Stop();
        SceneLoader.LoadScene( SceneNames.LEVEL_SELECT_SCENE );
    }

    public void AnimatorPlayStar1()
    {
        if ( starUIs.Length >= 1 )
        {
            starUIs[0].PlayAppear();
        }
    }

    public void AnimatorPlayStar2()
    {
        if ( starUIs.Length >= 2 )
        {
            starUIs[1].PlayAppear();
        }
    }

    public void AnimatorPlayStar3()
    {
        if ( starUIs.Length >= 3 )
        {
            starUIs[2].PlayAppear();
        }
    }
}
