﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using TMPro;

public class LevelStartButton : MonoBehaviour
{
    [Tooltip("We'll load this level number when our button is clicked on, but you need to hook up the callback.")]
    [SerializeField]
    protected int levelIndex;

    [Tooltip("All of our star panels, if any.")]
    [SerializeField]
    protected StarPanel[] starPanels;

    [Tooltip("The parent to our stars, we'll hide this if the level is locked.")]
    [SerializeField]
    protected GameObject starParent;

    [Tooltip("The text field for this level. We'll set the text appropriately if it's provided.")]
    [SerializeField]
    protected TextMeshProUGUI levelNumberText;

    [System.Serializable]
    public struct StarPanel
    {
        public GameObject empty;
        public GameObject full;
    }

    public void Start()
    {
        if ( levelNumberText != null )
        {
            bool isAvailable = levelIndex <= World.MaximumUnlockedLevel;
            
            if ( levelIndex >= 20 ) //hardcoded hack because World doesn't exist right now, really should fix
            {
                starParent.gameObject.SetActive( false );
                levelNumberText.text = World.MaximumUnlockedLevel >= 19 ? "!" : "?"; //also hack
            }
            else if ( isAvailable )
            {
                levelNumberText.text = "" + ( levelIndex + 1 );
                starParent.gameObject.SetActive( true );
                for ( int starPanelIndex = 0; starPanelIndex < starPanels.Length; starPanelIndex++ )
                {
                    starPanels[starPanelIndex].empty.SetActive( Star.GetMaximumStarCount( levelIndex ) <= starPanelIndex );
                    starPanels[starPanelIndex].full.SetActive( Star.GetMaximumStarCount( levelIndex ) > starPanelIndex );
                }
            }
            else
            {
                levelNumberText.text = "?";
                starParent.gameObject.SetActive( false );
            }
        }
    }

    public void LoadLevel()
    {
        bool isAvailable = levelIndex <= World.MaximumUnlockedLevel;

        if ( !isAvailable )
        {
            return;
        }

        World.LevelToLoad = levelIndex;

        if ( World.Instance == null )
        {
            SceneLoader.LoadScene( SceneNames.BOOTSTRAPPER_SCENE );
        }
        else
        {
            World.Instance.LoadFirstLevel();
        }
    }
}
