﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

[RequireComponent(typeof(Mover))]
public class EnemyController : MonoBehaviour
{
    [Tooltip("He's attracted to everything within this LayerMask.")]
    [SerializeField]
    protected LayerMask attractionLayerMask;

    [Tooltip("If a raycast to a trarget hits any of this, then he will not be attracted.")]
    [SerializeField]
    protected LayerMask blockAttractionThroughLayerMask;

    [Tooltip("If he's this close to an attractor, he'll move towards it.")]
    [SerializeField]
    protected float attractionRadius = 2.0f;

    [Tooltip("How quickly the character speeds up")]
    [SerializeField]
    protected float accelerationSpeed;

    [Tooltip("How quickly the character can move max")]
    [SerializeField]
    protected float maxSpeed;

    protected Collider2D[] circleCastResults = new Collider2D[64];
    protected Mover mover;

    protected virtual void Awake()
    {
        mover = GetComponent<Mover>();
    }

    protected virtual Collider2D FindClosestTargetInRange(Vector2 centerPoint, float radius, LayerMask layerMask)
    {
        int hitCount = Physics2D.OverlapCircleNonAlloc(centerPoint, radius, circleCastResults, layerMask);

        Collider2D closestCollider = null;
        float closestSqrDistance = 0.0f;

        for (int resultIndex = 0; resultIndex < hitCount; resultIndex++)
        {
            Collider2D coll = circleCastResults[resultIndex];
            Vector2 diff = ((Vector2)coll.transform.position - centerPoint);
            float sqrDistance = diff.sqrMagnitude;
            if (closestCollider == null || sqrDistance < closestSqrDistance)
            {
                if (!Physics2D.Raycast(centerPoint, diff.normalized, Mathf.Sqrt(sqrDistance), blockAttractionThroughLayerMask))
                {
                    closestSqrDistance = sqrDistance;
                    closestCollider = coll;
                }
            }
        }

        return closestCollider;
    }

    protected void MoveTowardsPoint(Vector2 moveTo)
    {
        Vector2 direction = (Vector3)moveTo - transform.position;

        float velMag = mover.InitialSpeed; //vel.magnitude;
        Vector2 velocity = Vector3.ClampMagnitude(mover.Velocity + direction * accelerationSpeed * Time.deltaTime, maxSpeed);

        mover.SetVelocity(velocity);
    }
}
