﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class PlayerView : CharacterView
{
    [Tooltip("This is our voice bubble, which we can use to say stuff.")]
    [SerializeField]
    protected VoiceBubbleView voiceBubbleView;

    [Tooltip("The sprite renderer for the object the player carries.")]
    [SerializeField]
    protected SpriteRenderer heldObjectSpriteRenderer;

    [Tooltip("These are the different things he says, both audio and string in the balloon.")]
    [SerializeField]
    protected EmoteList emotes;

    [Tooltip("How long after ... the player will speak on won level")]
    [SerializeField]
    protected float wonLevelEmotePause = 3f;

    [Tooltip("The sounds played upon grabbing a rune.")]
    [SerializeField]
    protected AudioClip[] gotRuneSounds;

    [Tooltip("The sounds played upon dropping a rune.")]
    [SerializeField]
    protected AudioClip[] dropRuneSounds;

    public EmoteList Emotes
    {
        get
        {
            return emotes;
        }
    }

    [System.Serializable]
    public struct EmoteList
    {
        public Emote[] wonLevel;
    }

    [System.Serializable]
    public struct Emote
    {
        public string text;
        public AudioClip audio;
    }

    public void SetHeldSprite(Sprite newSprite)
    {
        heldObjectSpriteRenderer.sprite = newSprite;
    }

    public override bool IsFacingRight
    {
        set
        {
            base.IsFacingRight = value;
            voiceBubbleView.TextIsFlipped = transform.localScale.x < 0;
        }
    }

    protected bool _hasWon;
    public virtual bool HasWon
    {
        get
        {
            return _hasWon;
        }
        set
        {
            if (value != _hasWon)
            {
                _hasWon = value;
                PlayEndEmote();
            }
        }
    }

    protected bool _hasRune;
    public bool HasRune
    {
        get
        {
            return _hasRune;
        }
        set
        {
            if ( _hasRune != value )
            {
                _hasRune = value;
                Sounds.Play( _hasRune ? gotRuneSounds : dropRuneSounds, audioSource );
            }
        }
    }

    public void PlayEndEmote()
    {
        voiceBubbleView.SetText("...");

        Invoke("CompleteEndEmote", wonLevelEmotePause);
    }

    public void CompleteEndEmote()
    {
        if (emotes.wonLevel.Length <= 0)
        {
            return;
        }

        int randomIndex = Random.Range(0, emotes.wonLevel.Length);

        Emote emote = emotes.wonLevel[randomIndex];

        voiceBubbleView.SetText(emote.text);
        Sounds.Play(emote.audio, audioSource);
    }
}
